# -*- coding: utf-8 -*-
import unittest

from skoolkittest import SkoolKitTestCase
from skoolkit.sftparser import SftParser

TEST_SFT = 'test-sft.sft'

TEST_SKOOL = """; Test processing of a data definition entry
d24576 DEFB 128

; Routine at 32768
;
; Test code disassembly with no comments.
;
; A Some value
; B Some other value
; @label=START
c32768 CPL
 32769 RET

; Test code disassembly with a comment marker but no comment
c32770 SCF          ;

; Test code disassembly with comments
c32771 CCF           ; This is a comment on a single line
*32772 HALT          ; This is a comment that spans two lines
                     ; and is followed by an empty comment line
                     ;

; Test the B directive
b32773 DEFB 0         ; Single byte
; Two pairs of bytes.
 32774 DEFB 0,0
 32776 DEFB 0,0
; Two triplets.
 32778 DEFB 0,0,0
 32781 DEFB 0,0,0

; Test the T directive
t32784 DEFM "Hello"

; Test the W directive
w32789 DEFW 0,0

; Test the Z directive
z32793 DEFS 10

; Test an invalid control directive
a32794 NOP

; Test a block ASM directive
; @rsub-begin
b32795 DEFB 0,0,0,0,0
; @rsub+else
 32795 DEFS 6
; @rsub+end
""".split('\n')

TEST_SKOOL_HEX = """; Test processing of a data definition entry
d24576 DEFB 128

; Routine at 32768
;
; Test code disassembly with no comments.
;
; A Some value
; B Some other value
; @label=START
c$8000 CPL
 $8001 RET

; Test code disassembly with a comment marker but no comment
c$8002 SCF          ;

; Test code disassembly with comments
c$8003 CCF           ; This is a comment on a single line
*$8004 HALT          ; This is a comment that spans two lines
                     ; and is followed by an empty comment line
                     ;

; Test the B directive
b$8005 DEFB $00       ; Single byte
; Two pairs of bytes.
 $8006 DEFB $00,$00
 $8008 DEFB $00,$00
; Two triplets.
 $800A DEFB $00,$00,$00
 $800D DEFB $00,$00,$00

; Test the T directive
t$8010 DEFM "Hello"

; Test the W directive
w$8015 DEFW $0000,$0000

; Test the Z directive
z$8019 DEFS $0A

; Test an invalid control directive
a32794 NOP

; Test a block ASM directive
; @rsub-begin
b$801B DEFB $00,$00,$00,$00,$00
; @rsub+else
 32795 DEFS 6
; @rsub+end
""".split('\n')

class SftParserTest(SkoolKitTestCase):
    def _parse_sft(self, asm_hex=False):
        snapshot = [0] * 65536
        snapshot[32768] = 47  # 32768 CPL
        snapshot[32769] = 201 # 32769 RET
        snapshot[32770] = 55  # 32770 SCF
        snapshot[32771] = 63  # 32771 CCF
        snapshot[32772] = 118 # 32772 HALT
        text = "Hello"
        snapshot[32784:32784 + len(text)] = [ord(c) for c in text]
        writer = SftParser(snapshot, TEST_SFT, asm_hex=asm_hex)
        writer.write_skool()
        return snapshot, self.out.getvalue().split('\n')

    def test_write_skool(self):
        snapshot, skool = self._parse_sft()
        self.assertEqual(len(skool), len(TEST_SKOOL))
        for i, line in enumerate(skool):
            self.assertEqual(line, TEST_SKOOL[i])
        self.assertEqual(snapshot[24576], 128)

    def test_write_skool_hex(self):
        snapshot, skool = self._parse_sft(True)
        self.assertEqual(len(skool), len(TEST_SKOOL_HEX))
        for i, line in enumerate(skool):
            self.assertEqual(line, TEST_SKOOL_HEX[i])
        self.assertEqual(snapshot[24576], 128)

if __name__ == '__main__':
    unittest.main()
