# -*- coding: utf-8 -*-
import unittest

from skoolkittest import SkoolKitTestCase
from skoolkit import skool2sft

USAGE = """Usage: skool2sft.py [options] FILE

  Convert a skool file into a skool file template, written to standard output.
  FILE may be a regular file, or '-' for standard input.

Options:
  -h  Write addresses in hexadecimal format"""

TEST_SKOOL = """; Test skool file for skool2sft testing
c32768 RET
"""

class Skool2SftTest(SkoolKitTestCase):
    def test_no_arguments(self):
        output, error = self.run_skool2sft(err_lines=True)
        self.assert_output_equal(error, USAGE.split('\n'))

    def test_invalid_arguments(self):
        for args in ('-h', '-x test.skool'):
            output, error = self.run_skool2sft(args, err_lines=True)
            self.assert_output_equal(error, USAGE.split('\n'))

    def test_no_options(self):
        skoolfile = 'test.skool'
        infile, write_hex = skool2sft.parse_args((skoolfile,))
        self.assertEqual(infile, skoolfile)
        self.assertFalse(write_hex)

    def test_option_h(self):
        skoolfile = 'test.skool'
        infile, write_hex = skool2sft.parse_args(('-h', skoolfile))
        self.assertEqual(infile, skoolfile)
        self.assertTrue(write_hex)

    def test_run(self):
        skoolfile = self.write_text_file(TEST_SKOOL, suffix='.skool')
        output, error = self.run_skool2sft(skoolfile)
        self.assertEqual(len(error), 0)
        self.assertEqual(len(output), 2)
        self.assertEqual(output[0], '; Test skool file for skool2sft testing')
        self.assertEqual(output[1], 'cC32768,1')

if __name__ == '__main__':
    unittest.main()
